/*
 * Copyright (C) 2007 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.brobwind.cubic;

import android.app.Activity;
import android.content.Context;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.view.WindowManager;

import java.nio.ShortBuffer;


public class CubicActivity extends Activity {
    private static final String TAG = "Cubic";
    private CubicView mCubic;

    private static class CubicView extends View {
        private Bitmap mBitmap;
        private Runnable mRun = new Runnable(){
                @Override
                public void run() {
                    CubicView.this.postDelayed(this, 40);

                    if (mBitmap != null) {
                        mBitmap.eraseColor(Color.rgb(0xe9, 0x96, 0x7a));
                        update(mBitmap);
                        invalidate();
                    }
                }
            };

        public CubicView(Context context) {
            super(context);

            Resources res = context.getResources();
            Drawable texDrawable = res.getDrawable(android.R.drawable.sym_def_app_icon);
            texDrawable.setBounds(0, 0, 32, 32);

            Bitmap texBitmap = Bitmap.createBitmap(32, 32, Bitmap.Config.RGB_565);
            texBitmap.eraseColor(Color.WHITE);
            Canvas texCanvas = new Canvas(texBitmap);
            texDrawable.draw(texCanvas);

            ShortBuffer texBuffer = ShortBuffer.allocate(32 * 32);
            texBitmap.copyPixelsToBuffer(texBuffer);
            setTex(texBuffer.array());

            texBitmap.recycle();

            setSystemUiVisibility(SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
                    | SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
                    | SYSTEM_UI_FLAG_LAYOUT_STABLE
                    | SYSTEM_UI_FLAG_LOW_PROFILE | SYSTEM_UI_FLAG_FULLSCREEN
                    | SYSTEM_UI_FLAG_HIDE_NAVIGATION | SYSTEM_UI_FLAG_IMMERSIVE_STICKY);
        }

        @Override
        protected void onSizeChanged(int w, int h, int oldw, int oldh) {
            super.onSizeChanged(w, h, oldw, oldh);

            if (w > 0 && h > 0) {
                if (mBitmap != null) {
                    if (w != mBitmap.getWidth() || h != mBitmap.getHeight()) {
                        mBitmap.recycle();
                        mBitmap = null;
                    }
                }
                if (mBitmap == null) {
                    mBitmap = Bitmap.createBitmap(w, h, Bitmap.Config.RGB_565);
                    mBitmap.eraseColor(Color.rgb(0xe9, 0x96, 0x7a));
                }
                setBounce(0, 0, w, h);
            }
        }

        @Override
        public void onDraw(Canvas canvas) {
            super.onDraw(canvas);

            if (mBitmap != null) {
                canvas.drawBitmap(mBitmap, 0, 0, null);
            }
        }

        public void onPause() {
            removeCallbacks(mRun);
        }

        public void onResume() {
            post(mRun);
        }
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        mCubic = new CubicView(this);
        setContentView(mCubic);

    }

    @Override
    protected void onPause() {
        super.onPause();
        mCubic.onPause();
    }

    @Override
    protected void onResume() {
        super.onResume();
        mCubic.onResume();
    }

    // ----------------------------------------------------
    static {
        System.loadLibrary("cubic-jni");
    }

    private static native void setTex(short tex[]);
    private static native void setBounce(int left, int top, int right, int bottom);
    private static native void update(Bitmap bitmap);
}
